# Copyright (C) 2016 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import BGP_FSM_ESTABLISHED, local
from lib.gobgp import GoBGPContainer
from lib.exabgp import ExaBGPContainer


class GoBGPTestBase(unittest.TestCase):

    wait_per_retry = 5
    retry_limit = 15

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g2 = GoBGPContainer(name='g2', asn=65001, router_id='192.168.0.2',
                            ctn_image_name=gobgp_ctn_image_name)
        e1 = ExaBGPContainer(name='e1', asn=65002, router_id='192.168.0.3')

        ctns = [g1, g2, e1]
        cls.clients = {cli.name: cli for cli in (g2, e1)}

        initial_wait_time = max(ctn.run() for ctn in ctns)
        time.sleep(initial_wait_time)

        for cli in list(cls.clients.values()):
            # Omit "passwd" to avoid a issue on ExaBGP version 4.0.5:
            # https://github.com/Exa-Networks/exabgp/issues/766
            g1.add_peer(cli, is_rs_client=True, passive=True, prefix_limit=10)
            cli.add_peer(g1)

        # advertise a route from route-server-clients
        g2.add_route('10.0.0.0/24')
        e1.add_route('10.0.1.0/24')

        cls.gobgp = g1

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        for cli in list(self.clients.values()):
            self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=cli)

    def test_02_add_neighbor(self):
        e2 = ExaBGPContainer(name='e2', asn=65001, router_id='192.168.0.4')
        time.sleep(e2.run())
        self.gobgp.add_peer(e2, is_rs_client=True)
        e2.add_peer(self.gobgp)

        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=e2)
        self.clients[e2.name] = e2

    def test_03_check_neighbor_rib(self):
        rib = self.gobgp.get_local_rib(self.clients['e2'])
        self.assertEqual(len(rib), 1)
        self.assertEqual(len(rib[0]['paths']), 1)
        path = rib[0]['paths'][0]
        self.assertTrue(65001 not in path['aspath'])

    def test_04_withdraw_path(self):
        self.clients['g2'].local('gobgp global rib del 10.0.0.0/24')
        time.sleep(1)
        afisafis = self.gobgp.get_neighbor(self.clients['g2'])['afi_safis']
        advertised = 0
        for afisafi in afisafis:
            s = afisafi.get('state')
            advertised += s.get('advertised')
            self.assertEqual(s.get('accepted'), None)  # means info['accepted'] == 0
            self.assertEqual(s.get('received'), None)  # means info['received'] == 0

        self.assertEqual(advertised, 1)





import pytest
from lib.noseplugin import parser_option


def pytest_addoption(parser):
    parser.addoption('--test-prefix', default='')
    parser.addoption('--gobgp-image', default='osrg/gobgp')
    parser.addoption('--exabgp-path', default='')
    parser.addoption('--go-path', default='')
    parser.addoption('--gobgp-log-level', default='info')
    parser.addoption('--test-index', type=int, default=0)
    parser.addoption('--config-format', default='yaml')


def pytest_configure(config):
    parser_option.test_prefix = config.getoption('--test-prefix')
    parser_option.gobgp_image = config.getoption('--gobgp-image')
    parser_option.exabgp_path = config.getoption('--exabgp-path')
    parser_option.go_path = config.getoption('--go-path')
    parser_option.gobgp_log_level = config.getoption('--gobgp-log-level')
    parser_option.test_index = config.getoption('--test-index')
    parser_option.config_format = config.getoption('--config-format')



# Copyright (C) 2016 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import json
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib import base
from lib.base import (
    BGP_FSM_ESTABLISHED,
    assert_several_times,
    local,
)
from lib.gobgp import GoBGPContainer
from lib.exabgp import ExaBGPContainer
from lib.noseplugin import parser_option


class GoBGPTestBase(unittest.TestCase):
    SEND_MAX = 5
    INSTALLED_PATHS = SEND_MAX + 1

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g2 = GoBGPContainer(name='g2', asn=65000, router_id='192.168.0.2',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g3 = GoBGPContainer(name='g3', asn=65000, router_id='192.168.0.3',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g4 = GoBGPContainer(name="g4", asn=65000, router_id="192.168.0.4",
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g5 = GoBGPContainer(name="g5", asn=65000,router_id="192.168.0.5",
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        e1 = ExaBGPContainer(name="e1", asn=65000, router_id="192.168.0.6")

        ctns = [g1, g2, g3, g4, g5, e1]
        initial_wait_time = max(ctn.run() for ctn in ctns)

        time.sleep(initial_wait_time)

        g1.add_peer(e1, addpath=16)
        e1.add_peer(g1, addpath=16)

        g1.add_peer(g2, is_rr_client=True)
        g2.add_peer(g1)

        g1.add_peer(g3, addpath=cls.SEND_MAX, is_rr_client=True)
        g3.add_peer(g1, addpath=cls.SEND_MAX)

        g4.add_peer(g5, addpath=cls.SEND_MAX)
        g5.add_peer(g4, addpath=cls.SEND_MAX)

        cls.g1 = g1
        cls.g2 = g2
        cls.g3 = g3
        cls.g4 = g4
        cls.g5 = g5
        cls.e1 = e1

    # test each neighbor state is turned establish
    def test_00_neighbor_established(self):
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g2)
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g3)
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.e1)
        self.g4.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g5)

    # prepare routes with path_id (no error check)
    def test_01_prepare_add_paths_routes(self):
        aspath = []
        for i in range(self.INSTALLED_PATHS):
            aspath.append((i + 1) * 100)
            self.e1.add_route(
                route="192.168.100.0/24",
                identifier=(i + 1) * 10,
                aspath=aspath,
            )

        # update multiple time the same route
        # and expect the route counter to increment only once
        for i in range(self.INSTALLED_PATHS):
            self.g4.add_route(
                route="192.168.100.0/24",
                identifier=10,
                local_pref=i + 100 + 1,
            )

    def test_02_check_g4_adj_out(self):
        def f():
            # the last update should have been received by g5
            rib = self.g4.get_adj_rib_out(self.g5, add_path_enabled=True)
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), 1)
            self.assertEqual(
                rib[0]["paths"][0]["local-pref"], 100 + self.INSTALLED_PATHS
            )
            self.assertFalse(rib[0]["paths"][0].get("send-max-filtered", False))

        assert_several_times(f)

    def test_03_check_g5_global_rib(self):
        def f():
            # the last update should have been received by g5
            rib = self.g5.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), 1)
            self.assertEqual(
                rib[0]["paths"][0]["local-pref"], 100 + self.INSTALLED_PATHS
            )

        assert_several_times(f)

    # test INSTALLED_PATHS routes are installed to the rib due to add-path feature
    def test_04_check_g1_global_rib(self):
        def f():
            rib = self.g1.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.INSTALLED_PATHS)

        assert_several_times(f)

    # test only the best path is advertised to g2
    def test_05_check_g2_global_rib(self):
        def f():
            rib = self.g2.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]['paths']), 1)
            self.assertEqual(len(rib[0]["paths"][0]["aspath"]), 1)

        assert_several_times(f)

    # test SEND_MAX routes are advertised to g3
    def test_06_check_g3_global_rib(self):
        def f():
            rib = self.g3.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.SEND_MAX)

        assert_several_times(f)

    def test_07_check_g1_adj_out(self):
        adj_out = self.g1.get_adj_rib_out(self.g2, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), 1)

        adj_out = self.g1.get_adj_rib_out(self.g3, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), self.INSTALLED_PATHS)
        # expect the last path to be filtered
        self.assertTrue(adj_out[0]["paths"][-1].get("send-max-filtered", False))

    # withdraw a route with path_id (no error check)
    def test_08_withdraw_route_with_path_id(self):
        self.e1.del_route(route="192.168.100.0/24", identifier=10)

    # test the withdrawn route is removed from the rib
    def test_09_check_g1_global_rib(self):
        def f():
            rib = self.g1.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.INSTALLED_PATHS - 1)
            # we deleted the highest priority path
            for path in rib[0]['paths']:
                self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    def test_10_check_g1_adj_out(self):
        adj_out = self.g1.get_adj_rib_out(self.g2, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), 1)

        adj_out = self.g1.get_adj_rib_out(self.g3, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), self.SEND_MAX)
        # expect the last path to not be filtered
        self.assertFalse(adj_out[0]["paths"][-1].get("send-max-filtered", False))

    # test the best path is replaced due to the removal from g1 rib
    def test_11_check_g2_global_rib(self):
        def f():
            rib = self.g2.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]['paths']), 1)
            self.assertEqual(len(rib[0]["paths"][0]["aspath"]), 2)

        assert_several_times(f)

    # test the withdrawn route is removed from the rib of g3
    # and the filtered route is advertised to g3
    def test_12_check_g3_global_rib(self):
        def f():
            rib = self.g3.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.SEND_MAX)
            for path in rib[0]['paths']:
                self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    # install a route with path_id via GoBGP CLI (no error check)
    def test_13_install_add_paths_route_via_cli(self):
        # identifier is duplicated with the identifier of the route from e1
        self.g1.add_route(route='192.168.100.0/24', identifier=10, local_pref=500)

    # test the route from CLI is installed to the rib
    def test_14_check_g1_global_rib(self):
        def f():
            rib = self.g1.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.INSTALLED_PATHS)
            for path in rib[0]['paths']:
                if not path["aspath"]:
                    self.assertEqual(path['local-pref'], 500)
                else:
                    self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    def test_15_check_g1_adj_out(self):
        adj_out = self.g1.get_adj_rib_out(self.g2, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), 1)

        adj_out = self.g1.get_adj_rib_out(self.g3, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), self.INSTALLED_PATHS)
        # the new best path shouldn't be advertised as it is added after
        # the limit is reached
        self.assertEqual(adj_out[0]["paths"][0]["local-pref"], 500)
        self.assertTrue(adj_out[0]["paths"][0].get("send-max-filtered", False))

    # test the best path is replaced due to the CLI route from g1 rib
    def test_16_check_g2_global_rib(self):
        def f():
            rib = self.g2.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]['paths']), 1)
            self.assertEqual(len(rib[0]["paths"][0]["aspath"]), 0)
            self.assertEqual(rib[0]["paths"][0]["local-pref"], 500)

        assert_several_times(f)

    # test the route from CLI is advertised from g1
    def test_17_check_g3_global_rib(self):
        def f():
            rib = self.g3.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.SEND_MAX)
            for path in rib[0]['paths']:
                self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    # remove non-existing route with path_id via GoBGP CLI (no error check)
    def test_18_remove_non_existing_add_paths_route_via_cli(self):
        # specify locally non-existing identifier which has the same value
        # with the identifier of the route from e1
        self.g1.del_route(route='192.168.100.0/24', identifier=20)

    # test none of route is removed by non-existing path_id via CLI
    def test_19_check_g1_global_rib(self):
        def f():
            rib = self.g1.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.INSTALLED_PATHS)
            for path in rib[0]['paths']:
                if not path["aspath"]:
                    self.assertEqual(path['local-pref'], 500)
                else:
                    self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    # remove route with path_id via GoBGP CLI (no error check)
    def test_20_remove_add_paths_route_via_cli(self):
        self.g1.del_route(route='192.168.100.0/24', identifier=10)

    def test_21_check_g1_adj_out(self):
        adj_out = self.g1.get_adj_rib_out(self.g2, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), 1)

        adj_out = self.g1.get_adj_rib_out(self.g3, add_path_enabled=True)
        self.assertEqual(len(adj_out), 1)
        self.assertEqual(len(adj_out[0]["paths"]), self.INSTALLED_PATHS - 1)

    # test the route is removed from the rib via CLI
    def test_22_check_g1_global_rib(self):
        def f():
            rib = self.g1.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.INSTALLED_PATHS - 1)
            for path in rib[0]['paths']:
                if not path["aspath"]:
                    self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)

    # test the best path is replaced the removal from g1 rib
    def test_23_check_g2_global_rib(self):
        def f():
            rib = self.g2.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]['paths']), 1)
            self.assertEqual(len(rib[0]["paths"][0]["aspath"]), 2)

        assert_several_times(f)

    # test the removed route from CLI is withdrawn by g1
    def test_24_check_g3_global_rib(self):
        def f():
            rib = self.g3.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]["paths"]), self.SEND_MAX)
            for path in rib[0]['paths']:
                if not path["aspath"]:
                    self.assertTrue(2 <= len(path["aspath"]) <= self.INSTALLED_PATHS)

        assert_several_times(f)





# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest
import sys
import time

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    BGP_FSM_IDLE,
    BGP_FSM_ESTABLISHED,
    local,
)
from lib.gobgp import GoBGPContainer
from lib.exabgp import ExaBGPContainer


class GoBGPTestBase(unittest.TestCase):

    wait_per_retry = 5
    retry_limit = 10

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)

        rs_clients = [
            ExaBGPContainer(name='q{0}'.format(i + 1), asn=(65001 + i),
                            router_id='192.168.0.{0}'.format(i + 2))
            for i in range(4)]
        ctns = [g1] + rs_clients

        initial_wait_time = max(ctn.run() for ctn in ctns)
        time.sleep(initial_wait_time)

        for i, rs_client in enumerate(rs_clients):
            g1.add_peer(rs_client, is_rs_client=True)
            as2 = False
            if i > 1:
                as2 = True
            rs_client.add_peer(g1, as2=as2)

        # advertise a route from route-server-clients
        for idx, rs_client in enumerate(rs_clients):
            route = '10.0.{0}.0/24'.format(idx + 1)
            rs_client.add_route(route)
            if idx < 2:
                route = '10.0.10.0/24'
            rs_client.add_route(route)

        cls.gobgp = g1
        cls.quaggas = {x.name: x for x in rs_clients}

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        for q in self.quaggas.values():
            self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q)

    def test_02_check_gobgp_local_rib(self):
        for rs_client in self.quaggas.values():
            done = False
            for _ in range(self.retry_limit):
                if done:
                    break

                state = self.gobgp.get_neighbor_state(rs_client)
                self.assertEqual(state, BGP_FSM_ESTABLISHED)
                local_rib = self.gobgp.get_local_rib(rs_client)
                if len(local_rib) < (len(self.quaggas) - 1):
                    time.sleep(self.wait_per_retry)
                    continue

                self.assertEqual(len(local_rib), 4)
                done = True

            if done:
                continue
            # should not reach here
            raise AssertionError

    def test_03_stop_q2_and_check_neighbor_status(self):
        q2 = self.quaggas['q2']
        q2.remove()
        self.gobgp.wait_for(expected_state=BGP_FSM_IDLE, peer=q2)





# Copyright (C) 2017 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    assert_several_times,
    Bridge,
    BGP_FSM_ESTABLISHED,
    local,
)
from lib.gobgp import GoBGPContainer


class ZebraNHTTest(unittest.TestCase):
    """
    Test case for Next-Hop Tracking (NHT) with Zebra integration.

    Verifies that GoBGP correctly reacts to NEXTHOP_UPDATE messages from
    zebra by marking paths as reachable or unreachable, and propagates
    the resulting advertise/withdraw to BGP peers.

    OSPF is intentionally not used: nexthop reachability is driven by
    adding or removing a static route in r2's zebra, which is
    deterministic and keeps the test free of IGP convergence timing.

    Note: Quagga static routes always carry metric=0, so MED values are
    not validated here; MED handling is covered by unit tests.
    """

    # R1: GoBGP
    # R2: GoBGP + Zebra (static routes)
    #
    # +----+      +----+
    # | R1 |------| R2 |
    # +----+      +----+

    NEXTHOP = '10.3.1.1'
    PREFIX = '10.3.1.0/24'

    def _set_static_nexthop(self):
        # Point the static route at the r1-r2 bridge so zebra treats
        # the nexthop as reachable.
        self.r2.local(
            "vtysh -c 'configure terminal'"
            " -c 'ip route %s/32 192.168.12.1'"
            % self.NEXTHOP)

    def _remove_static_nexthop(self):
        self.r2.local(
            "vtysh -c 'configure terminal'"
            " -c 'no ip route %s/32 192.168.12.1'"
            % self.NEXTHOP)

    def _assert_best(self, rt, prefix):
        # Path is present and best ("*>").
        self.assertEqual(rt.local(
            "gobgp global rib -a ipv4 %s"
            " | grep '^\\*>' > /dev/null"
            " && echo OK || echo NG" % prefix,
            capture=True), 'OK')

    def _assert_not_best(self, rt, prefix):
        # Path is present but not best ("* ", not "*>").
        self.assertEqual(rt.local(
            "gobgp global rib -a ipv4 %s"
            " | grep '^\\* ' > /dev/null"
            " && echo OK || echo NG" % prefix,
            capture=True), 'OK')

    def _assert_no_prefix(self, rt, prefix):
        # Prefix is not in the table at all.
        self.assertEqual(rt.local(
            "gobgp global rib -a ipv4 %s"
            " | grep 'Network not in table' > /dev/null"
            " && echo OK || echo NG" % prefix,
            capture=True), 'OK')

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix
        cls.r1 = GoBGPContainer(
            name='r1', asn=65000, router_id='192.168.0.1',
            ctn_image_name=gobgp_ctn_image_name,
            log_level=parser_option.gobgp_log_level,
            zebra=False)

        cls.r2 = GoBGPContainer(
            name='r2', asn=65000, router_id='192.168.0.2',
            ctn_image_name=gobgp_ctn_image_name,
            log_level=parser_option.gobgp_log_level,
            zebra=True,
            zapi_version=3)

        wait_time = max(ctn.run() for ctn in [cls.r1, cls.r2])
        time.sleep(wait_time)

        cls.br_r1_r2 = Bridge(name='br_r1_r2', subnet='192.168.12.0/24')
        for ctn in (cls.r1, cls.r2):
            cls.br_r1_r2.addif(ctn)

    def test_01_BGP_neighbor_established(self):
        # Test to start BGP connection up between r1-r2.

        self.r1.add_peer(self.r2, bridge=self.br_r1_r2.name)
        self.r2.add_peer(self.r1, bridge=self.br_r1_r2.name)

        self.r1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.r2)

    def test_02_reachable_nexthop(self):
        # With the nexthop reachable via a static route, adding a BGP
        # path with that nexthop must result in the path being best on
        # r2 and reaching r1.
        self._set_static_nexthop()

        self.r2.local(
            'gobgp global rib add -a ipv4 %s nexthop %s'
            % (self.PREFIX, self.NEXTHOP))

        assert_several_times(
            f=lambda: self._assert_best(self.r2, self.PREFIX), t=60)
        assert_several_times(
            f=lambda: self._assert_best(self.r1, self.PREFIX), t=60)

    def test_03_nexthop_unreachable(self):
        # Remove the static route: nexthop becomes unreachable.
        # The existing path must transition from best to not-best on r2
        # and be withdrawn from r1. This exercises the
        # valid -> invalid transition path in GetChanges.
        self._remove_static_nexthop()

        assert_several_times(
            f=lambda: self._assert_not_best(self.r2, self.PREFIX), t=60)
        assert_several_times(
            f=lambda: self._assert_no_prefix(self.r1, self.PREFIX), t=60)

    def test_04_nexthop_restore(self):
        # Re-add the static route. The existing path must become best
        # again on r2 and be re-advertised to r1. This exercises the
        # invalid -> valid revalidation path.
        self._set_static_nexthop()

        assert_several_times(
            f=lambda: self._assert_best(self.r2, self.PREFIX), t=60)
        assert_several_times(
            f=lambda: self._assert_best(self.r1, self.PREFIX), t=60)

    def test_05_add_path_while_unreachable(self):
        # Make the nexthop unreachable again, then add a brand-new
        # path. The new path must be tracked as not-best on r2 and
        # must never be advertised to r1. This exercises the separate
        # code path where nexthop cache is applied to a newly added
        # path before propagation.
        self._remove_static_nexthop()

        prefix = '10.3.2.0/24'
        self.r2.local(
            'gobgp global rib add -a ipv4 %s nexthop %s'
            % (prefix, self.NEXTHOP))

        assert_several_times(
            f=lambda: self._assert_not_best(self.r2, prefix), t=60)
        assert_several_times(
            f=lambda: self._assert_no_prefix(self.r1, prefix), t=60)

    def test_06_restore_revalidates_new_path(self):
        # Re-add the static route. The new path added while unreachable
        # must now become best and be advertised to r1, confirming the
        # newly-added-invalid path can be revalidated.
        self._set_static_nexthop()

        assert_several_times(
            f=lambda: self._assert_best(self.r2, '10.3.2.0/24'), t=60)
        assert_several_times(
            f=lambda: self._assert_best(self.r1, '10.3.2.0/24'), t=60)





class _ParserOption:
    test_prefix = ""
    gobgp_image = "osrg/gobgp"
    exabgp_path = ""
    go_path = ""
    gobgp_log_level = "info"
    test_index = 0
    config_format = "yaml"


parser_option = _ParserOption()



# Copyright (C) 2017 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import BGP_FSM_ESTABLISHED, local
from lib.gobgp import GoBGPContainer
from lib.exabgp import ExaBGPContainer


class GoBGPTestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        e1 = ExaBGPContainer(name='e1', asn=65001, router_id='192.168.0.2')

        ctns = [g1, e1]
        initial_wait_time = max(ctn.run() for ctn in ctns)
        time.sleep(initial_wait_time)

        g1.add_peer(e1, treat_as_withdraw=True)
        e1.add_peer(g1)

        cls.g1 = g1
        cls.e1 = e1

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.e1)

    def test_02_attribute_discard(self):
        # Malformed attribute 'AGGREGATOR' should be discard, but the session should not be disconnected.
        self.e1.add_route('10.0.0.0/24', attribute='0x07 0xc0 0x0000006400')

        # Confirm the session is not disconnected
        for _ in range(5):
            state = self.g1.get_neighbor_state(self.e1)
            self.assertTrue(BGP_FSM_ESTABLISHED, state)
            time.sleep(1)

        # Confirm the path is added
        dests = self.g1.get_global_rib()
        self.assertEqual(len(dests), 1)
        routes = dests[0]['paths']
        self.assertEqual(len(routes), 1)

        # Confirm the attribute 'AGGREGATOR(type=7)' is discarded
        for d in routes[0]['attrs']:
            self.assertFalse(d['type'] == 7)

        self.e1.del_route('10.0.0.0/24')

    def test_03_treat_as_withdraw(self):
        # Malformed attribute 'MULTI_EXIT_DESC' should be treated as withdraw,
        # but the session should not be disconnected.
        self.e1.add_route('20.0.0.0/24', attribute='0x04 0x80 0x00000064')
        self.e1.add_route('30.0.0.0/24', attribute='0x04 0x80 0x00000064')
        # Malformed
        self.e1.add_route('30.0.0.0/24', attribute='0x04 0x80 0x0000000064')

        # Confirm the session is not disconnected
        for _ in range(5):
            state = self.g1.get_neighbor_state(self.e1)
            self.assertTrue(BGP_FSM_ESTABLISHED, state)
            time.sleep(1)

        # Confirm the number of path in RIB is only one
        dests = self.g1.get_global_rib()
        self.assertEqual(len(dests), 1)
        self.assertEqual(dests[0]['paths'][0]['nlri']['prefix'], '20.0.0.0/24')





# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    Bridge,
    BGP_FSM_ESTABLISHED,
    local,
)
from lib.gobgp import GoBGPContainer
from lib.quagga import QuaggaBGPContainer


class GoBGPTestBase(unittest.TestCase):
    # The same topology for IPv4 and IPv6 environment.
    # o1: Quagga (not router)
    # g1: GoBGP
    # q1: Quagga (Zebra + BGPd)
    # o2: Quagga (not router)
    #
    # +----+       +----+           +----+       +----+
    # | o1 |       | g1 |===(BGP)===| q1 |       | o2 |
    # +----+       +----+           +----+       +----+
    #    |          |  |             |  |          |
    #    +--(br01)--+  +---(br02)----+  +--(br03)--+

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        # preparing the container for ipv4
        g1_v4 = GoBGPContainer(name='g1_v4', asn=65000, router_id='192.168.0.1',
                               ctn_image_name=gobgp_ctn_image_name,
                               log_level=parser_option.gobgp_log_level,
                               zebra=True)
        q1_v4 = QuaggaBGPContainer(name='q1_v4', asn=65001, router_id='192.168.0.2', zebra=True)
        o1_v4 = QuaggaBGPContainer(name='o1_v4', asn=65002, router_id='192.168.0.3')
        o2_v4 = QuaggaBGPContainer(name='o2_v4', asn=65002, router_id='192.168.0.4')

        # preparing the container for ipv6
        g1_v6 = GoBGPContainer(name='g1_v6', asn=65000, router_id='192.168.0.1',
                               ctn_image_name=gobgp_ctn_image_name,
                               log_level=parser_option.gobgp_log_level,
                               zebra=True)
        q1_v6 = QuaggaBGPContainer(name='q1_v6', asn=65001, router_id='192.168.0.2', zebra=True)
        o1_v6 = QuaggaBGPContainer(name='o1_v6', asn=65002, router_id='192.168.0.3')
        o2_v6 = QuaggaBGPContainer(name='o2_v6', asn=65002, router_id='192.168.0.4')

        # preparing the bridge for ipv4
        br01_v4 = Bridge(name='br01_v4', subnet='192.168.10.0/24')
        br02_v4 = Bridge(name='br02_v4', subnet='192.168.20.0/24')
        br03_v4 = Bridge(name='br03_v4', subnet='192.168.30.0/24')

        # preparing the bridge for ipv6
        br01_v6 = Bridge(name='br01_v6', subnet='2001:10::/32')
        br02_v6 = Bridge(name='br02_v6', subnet='2001:20::/32')
        br03_v6 = Bridge(name='br03_v6', subnet='2001:30::/32')

        cls.ctns = {
            'ipv4': [g1_v4, q1_v4, o1_v4, o2_v4],
            'ipv6': [g1_v6, q1_v6, o1_v6, o2_v6],
        }
        cls.gobgps = {
            'ipv4': g1_v4,
            'ipv6': g1_v6,
        }
        cls.quaggas = {
            'ipv4': q1_v4,
            'ipv6': q1_v6,
        }
        cls.others = {
            'ipv4': [o1_v4, o2_v4],
            'ipv6': [o1_v6, o2_v6],
        }
        cls.bridges = {
            'br01_v4': br01_v4,
            'br02_v4': br02_v4,
            'br03_v4': br03_v4,
            'br01_v6': br01_v6,
            'br02_v6': br02_v6,
            'br03_v6': br03_v6,
        }

    """
      No.1 start up ipv4 containers and check state
           each neighbor is established in ipv4 environment
    """

    def test_01_check_neighbor_established(self):
        g1 = self.gobgps['ipv4']
        q1 = self.quaggas['ipv4']
        o1 = self.others['ipv4'][0]
        o2 = self.others['ipv4'][1]
        # start up containers of ipv4 environment
        initial_wait_time = max(ctn.run() for ctn in self.ctns['ipv4'])
        time.sleep(initial_wait_time)

        # make ipv4 bridge and set ip to each container
        [self.bridges['br01_v4'].addif(ctn) for ctn in [o1, g1]]
        [self.bridges['br02_v4'].addif(ctn) for ctn in [g1, q1]]
        [self.bridges['br03_v4'].addif(ctn) for ctn in [q1, o2]]

        g1.add_peer(q1, bridge=self.bridges['br02_v4'].name)
        q1.add_peer(g1, bridge=self.bridges['br02_v4'].name)

        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q1)

    """
      No.2 check whether the ping is reachable in container
           that have previously beyond the GoBGP in ipv4 environment
    """

    def test_02_check_reachability_between_o1_and_o2(self):
        g1 = self.gobgps['ipv4']
        q1 = self.quaggas['ipv4']
        o1 = self.others['ipv4'][0]
        o2 = self.others['ipv4'][1]

        # set o1's default gateway as g1
        g1_addr = g1.ip_addrs[1][1].split('/')[0]
        o1.add_static_route(self.bridges['br03_v4'].subnet, g1_addr)

        # set o2's default gateway as q1
        q1_addr = q1.ip_addrs[2][1].split('/')[0]
        o2.add_static_route(self.bridges['br01_v4'].subnet, q1_addr)

        # test reachability between o1 and o2
        addrs = [e[1] for e in o2.ip_addrs if 'br03_v4' in e[2]]
        self.assertEqual(len(addrs), 1)
        o2_addr = addrs[0]
        o1.get_reachability(o2_addr)

    """
      No.4 start up ipv6 containers and check state
           each neighbor is established in ipv6 environment
    """

    def test_04_check_neighbor_established_v6(self):
        g1 = self.gobgps['ipv6']
        q1 = self.quaggas['ipv6']
        o1 = self.others['ipv6'][0]
        o2 = self.others['ipv6'][1]
        # start up containers of ipv6 environment
        initial_wait_time = max(ctn.run() for ctn in self.ctns['ipv6'])
        time.sleep(initial_wait_time)

        # make ipv6 bridge and set ip to each container
        [self.bridges['br01_v6'].addif(ctn) for ctn in [o1, g1]]
        [self.bridges['br02_v6'].addif(ctn) for ctn in [g1, q1]]
        [self.bridges['br03_v6'].addif(ctn) for ctn in [q1, o2]]

        g1.add_peer(q1, bridge=self.bridges['br02_v6'].name, v6=True)
        q1.add_peer(g1, bridge=self.bridges['br02_v6'].name, v6=True)

        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q1)

    """
      No.5 check whether the ping is reachable in container
           that have previously beyond the GoBGP in ipv6 environment
    """

    def test_05_check_reachability_between_o1_and_o2(self):
        g1 = self.gobgps['ipv6']
        q1 = self.quaggas['ipv6']
        o1 = self.others['ipv6'][0]
        o2 = self.others['ipv6'][1]

        # set o1's default gateway as g1
        g1_addr = [a for a in g1.ip6_addrs if a[2] == self.bridges['br01_v6'].name][0][1].split('/')[0]
        o1.add_static_route(self.bridges['br03_v6'].subnet, g1_addr)

        # set o2's default gateway as q1
        q1_addr = [a for a in q1.ip6_addrs if a[2] == self.bridges['br03_v6'].name][0][1].split('/')[0]
        o2.add_static_route(self.bridges['br01_v6'].subnet, q1_addr)

        # test reachability between o1 and o2
        addrs = [e[1] for e in o2.ip6_addrs if 'br03_v6' in e[2]]
        self.assertEqual(len(addrs), 1)
        o2_addr = addrs[0]
        o1.get_reachability(o2_addr)

    def test_07_mpath_test_setup(self):
        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=parser_option.gobgp_image,
                            log_level=parser_option.gobgp_log_level,
                            config_format=parser_option.config_format,
                            zebra=True)
        g2 = GoBGPContainer(name='g2', asn=65001, router_id='192.168.0.2',
                            ctn_image_name=parser_option.gobgp_image)
        g3 = GoBGPContainer(name='g3', asn=65001, router_id='192.168.0.3',
                            ctn_image_name=parser_option.gobgp_image)
        g4 = GoBGPContainer(name='g4', asn=65000, router_id='192.168.0.4',
                            ctn_image_name=parser_option.gobgp_image)
        g5 = GoBGPContainer(name='g5', asn=65000, router_id='192.168.0.5',
                            ctn_image_name=parser_option.gobgp_image)

        ctns = [g1, g2, g3, g4, g5]
        for ctn in ctns:
            self.ctns[ctn.name] = ctn

        initial_wait_time = max(ctn.run() for ctn in ctns)

        time.sleep(initial_wait_time)

        # advertise same prefix
        g2.add_route('10.0.10.0/24')
        g3.add_route('10.0.10.0/24')
        g4.add_route('10.0.10.0/24')
        g5.add_route('10.0.10.0/24')

        for g in [g2, g3, g4, g5]:
            g1.add_peer(g)
            g.add_peer(g1)

    def test_08_mpath_test_check_neighbor_established(self):
        g1 = self.ctns['g1']
        g2 = self.ctns['g2']
        g3 = self.ctns['g3']
        g4 = self.ctns['g4']
        g5 = self.ctns['g5']
        for g in [g2, g3, g4, g5]:
            g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g)

    def test_09_mpath_test_check_mpath_injected(self):
        g1 = self.ctns['g1']
        g2 = self.ctns['g2']
        g3 = self.ctns['g3']
        g4 = self.ctns['g4']
        g5 = self.ctns['g5']

        def nexthops():
            n = []
            for line in g1.local('ip route show 10.0.10.0/24', capture=True).split('\n'):
                line = line.strip()
                if 'via' in line:
                    n.append(line.split(' ')[2].strip())
            return n

        def validate_nexthops(peers):
            interval = 1
            count = 0
            timeout = 30
            while True:
                valid = False
                nhs = nexthops()
                if len(nhs) == len(peers):
                    valid = True
                    for peer in peers:
                        if g1.peers[peer]['neigh_addr'].split('/')[0] not in nhs:
                            valid = False
                            break
                if valid:
                    return

                time.sleep(interval)
                count += interval
                if count >= timeout:
                    raise Exception(nhs)

        validate_nexthops([g4, g5])

        g4.local('gobgp g ri del 10.0.10.0/24')
        validate_nexthops([g5])

        g4.local('gobgp g ri add 10.0.10.0/24 local-pref 200')
        validate_nexthops([g4])

        g4.local('gobgp g ri del 10.0.10.0/24')
        g5.local('gobgp g ri del 10.0.10.0/24')
        validate_nexthops([g2, g3])

        g3.local('gobgp g ri del 10.0.10.0/24')
        validate_nexthops([g2])

        g3.local('gobgp g ri add 10.0.10.0/24 med 10')
        validate_nexthops([g2])

        g2.local('gobgp g ri add 10.0.10.0/24 med 20')
        validate_nexthops([g3])





# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import json
import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    BGP_FSM_IDLE,
    BGP_FSM_ACTIVE,
    BGP_FSM_ESTABLISHED,
    BGP_ATTR_TYPE_MULTI_EXIT_DISC,
    BGP_ATTR_TYPE_LOCAL_PREF,
    wait_for_completion,
    assert_several_times,
    local,
)
from lib.gobgp import (
    GoBGPContainer,
    extract_path_attribute,
)
from lib.quagga import QuaggaBGPContainer
from lib.exabgp import ExaBGPContainer


class GoBGPTestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        q1 = QuaggaBGPContainer(name='q1', asn=65001, router_id='192.168.0.2')
        q2 = QuaggaBGPContainer(name='q2', asn=65002, router_id='192.168.0.3')
        q3 = QuaggaBGPContainer(name='q3', asn=65003, router_id='192.168.0.4')

        qs = [q1, q2, q3]
        ctns = [g1, q1, q2, q3]

        initial_wait_time = max(ctn.run() for ctn in ctns)
        time.sleep(initial_wait_time)

        for q in qs:
            g1.add_peer(q, passwd='passwd')
            q.add_peer(g1, passwd='passwd', passive=True)

        # advertise a route from q1, q2, q3
        for idx, q in enumerate(qs):
            route = '10.0.{0}.0/24'.format(idx + 1)
            q.add_route(route)

        cls.gobgp = g1
        cls.quaggas = {'q1': q1, 'q2': q2, 'q3': q3}

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        for q in self.quaggas.values():
            self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q)

    def test_02_check_gobgp_global_rib(self):
        for q in self.quaggas.values():
            # paths expected to exist in gobgp's global rib
            routes = list(q.routes.keys())
            timeout = 120
            interval = 1
            count = 0

            while True:
                # gobgp's global rib
                state = self.gobgp.get_neighbor_state(q)
                self.assertEqual(state, BGP_FSM_ESTABLISHED)
                global_rib = [p['prefix'] for p in self.gobgp.get_global_rib()]

                for p in global_rib:
                    if p in routes:
                        routes.remove(p)

                if len(routes) == 0:
                    break

                time.sleep(interval)
                count += interval
                if count >= timeout:
                    raise Exception('timeout')

    # check gobgp properly add it's own asn to aspath
    def test_03_check_gobgp_adj_out_rib(self):
        for q in self.quaggas.values():
            for path in self.gobgp.get_adj_rib_out(q):
                asns = path['aspath']
                self.assertTrue(self.gobgp.asn in asns)

    # check routes are properly advertised to all BGP speaker
    def test_04_check_quagga_global_rib(self):
        interval = 1
        timeout = int(120 / interval)
        for q in self.quaggas.values():
            done = False
            for _ in range(timeout):
                if done:
                    break
                global_rib = q.get_global_rib()
                global_rib = [p['prefix'] for p in global_rib]
                if len(global_rib) < len(self.quaggas):
                    time.sleep(interval)
                    continue

                self.assertEqual(len(global_rib), len(self.quaggas))

                for c in self.quaggas.values():
                    for r in c.routes:
                        self.assertTrue(r in global_rib)
                done = True
            if done:
                continue
            # should not reach here
            raise AssertionError

    def test_05_add_quagga(self):
        q4 = QuaggaBGPContainer(name='q4', asn=65004, router_id='192.168.0.5')
        self.quaggas['q4'] = q4
        initial_wait_time = q4.run()
        time.sleep(initial_wait_time)

        self.gobgp.add_peer(q4)
        q4.add_peer(self.gobgp)

        q4.add_route('10.0.4.0/24')

        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q4)

    def test_06_check_global_rib(self):
        self.test_02_check_gobgp_global_rib()
        self.test_04_check_quagga_global_rib()

    def test_07_stop_one_quagga(self):
        g1 = self.gobgp
        q4 = self.quaggas['q4']
        q4.stop()
        self.gobgp.wait_for(expected_state=BGP_FSM_ACTIVE, peer=q4)

        g1.del_peer(q4)
        del self.quaggas['q4']

    # check gobgp properly send withdrawal message with q4's route
    def test_08_check_global_rib(self):
        self.test_02_check_gobgp_global_rib()
        self.test_04_check_quagga_global_rib()

    def test_09_add_distant_relative(self):
        q1 = self.quaggas['q1']
        q2 = self.quaggas['q2']
        q3 = self.quaggas['q3']
        q5 = QuaggaBGPContainer(name='q5', asn=65005, router_id='192.168.0.6')

        initial_wait_time = q5.run()
        time.sleep(initial_wait_time)

        for q in [q2, q3]:
            q5.add_peer(q)
            q.add_peer(q5)

        med200 = {'name': 'med200',
                  'type': 'permit',
                  'match': '0.0.0.0/0',
                  'med': 200}
        q2.add_policy(med200, self.gobgp, 'out')
        med100 = {'name': 'med100',
                  'type': 'permit',
                  'match': '0.0.0.0/0',
                  'med': 100}
        q3.add_policy(med100, self.gobgp, 'out')

        q5.add_route('10.0.6.0/24')

        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q2)
        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q3)
        q2.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q5)
        q3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q5)

        timeout = 120
        interval = 1
        count = 0
        while True:
            paths = self.gobgp.get_adj_rib_out(q1, '10.0.6.0/24')
            if len(paths) > 0:
                path = paths[0]
                print("{0}'s nexthop is {1}".format(path['nlri']['prefix'],
                                                    path['nexthop']))
                n_addrs = [i[1].split('/')[0] for i in self.gobgp.ip_addrs]
                if path['nexthop'] in n_addrs:
                    break

            time.sleep(interval)
            count += interval
            if count >= timeout:
                raise Exception('timeout')

    def test_10_originate_path(self):
        self.gobgp.add_route('10.10.0.0/24')
        dst = self.gobgp.get_global_rib('10.10.0.0/24')
        self.assertEqual(len(dst), 1)
        self.assertEqual(len(dst[0]['paths']), 1)
        path = dst[0]['paths'][0]
        self.assertEqual(path['nexthop'], '0.0.0.0')
        self.assertEqual(len(path['aspath']), 0)

    def test_11_check_adj_rib_out(self):
        for q in self.quaggas.values():
            paths = self.gobgp.get_adj_rib_out(q, '10.10.0.0/24')
            self.assertEqual(len(paths), 1)
            path = paths[0]
            peer_info = self.gobgp.peers[q]
            local_addr = peer_info['local_addr'].split('/')[0]
            self.assertEqual(path['nexthop'], local_addr)
            self.assertEqual(path['aspath'], [self.gobgp.asn])

    def test_12_disable_peer(self):
        q1 = self.quaggas['q1']
        self.gobgp.disable_peer(q1)
        self.gobgp.wait_for(expected_state=BGP_FSM_IDLE, peer=q1)

        time.sleep(3)

        for route in q1.routes.keys():
            dst = self.gobgp.get_global_rib(route)
            self.assertEqual(len(dst), 0)

            for q in self.quaggas.values():
                if q is q1:
                    continue
                paths = self.gobgp.get_adj_rib_out(q, route)
                self.assertEqual(len(paths), 0)

    def test_13_enable_peer(self):
        q1 = self.quaggas['q1']
        self.gobgp.enable_peer(q1)
        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q1)

    def test_14_check_adj_rib_out(self):
        self.test_11_check_adj_rib_out()

    def test_15_check_active_connection(self):
        g1 = self.gobgp
        g2 = GoBGPContainer(name='g2', asn=65000, router_id='192.168.0.7',
                            ctn_image_name=self.gobgp.image,
                            log_level=parser_option.gobgp_log_level)
        time.sleep(g2.run())
        self.quaggas['g2'] = g2
        g2.add_peer(g1, passive=True)
        g1.add_peer(g2)
        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g2)

    def test_16_check_local_pref_and_med_handling(self):
        g1 = self.gobgp
        g1.add_route('10.20.0.0/24', local_pref=1000, med=2000)
        # iBGP peer
        g2 = self.quaggas['g2']
        paths = g2.get_global_rib('10.20.0.0/24')
        self.assertEqual(len(paths), 1)
        self.assertEqual(len(paths[0]['paths']), 1)
        path = paths[0]['paths'][0]
        local_pref = extract_path_attribute(path, BGP_ATTR_TYPE_LOCAL_PREF)
        self.assertEqual(local_pref['value'], 1000)
        med = extract_path_attribute(path, BGP_ATTR_TYPE_MULTI_EXIT_DISC)
        self.assertEqual(med['metric'], 2000)

        # eBGP peer
        q1 = self.quaggas['q1']
        paths = q1.get_global_rib('10.20.0.0/24')
        self.assertEqual(len(paths), 1)
        path = paths[0]
        local_pref = extract_path_attribute(path, BGP_ATTR_TYPE_LOCAL_PREF)
        # local_pref's default value is 100
        self.assertEqual(local_pref['value'], 100)
        med = extract_path_attribute(path, BGP_ATTR_TYPE_MULTI_EXIT_DISC)
        self.assertEqual(med['metric'], 2000)

    def test_17_check_shutdown(self):
        g1 = self.gobgp
        q1 = self.quaggas['q1']
        q2 = self.quaggas['q2']
        q3 = self.quaggas['q3']

        q2.add_route('20.0.0.0/24')
        q3.add_route('20.0.0.0/24')

        self.test_01_neighbor_established()

        self.test_02_check_gobgp_global_rib()

        paths = q1.get_global_rib('20.0.0.0/24')
        self.assertEqual(len(paths), 1)
        n_addrs = [i[1].split('/')[0] for i in self.gobgp.ip_addrs]
        self.assertIn(paths[0]['nexthop'], n_addrs)

        q3.stop()

        self.gobgp.wait_for(expected_state=BGP_FSM_ACTIVE, peer=q3)

        def f():
            paths = q1.get_global_rib('20.0.0.0/24')
            self.assertEqual(len(paths), 1)
            self.assertIn(paths[0]['nexthop'], n_addrs)

        assert_several_times(f)

        g1.del_peer(q3)
        del self.quaggas['q3']

    def test_18_check_withdrawal(self):
        g1 = self.gobgp
        q1 = self.quaggas['q1']
        q2 = self.quaggas['q2']

        g1.add_route('30.0.0.0/24')
        q1.add_route('30.0.0.0/24')

        self.test_01_neighbor_established()

        self.test_02_check_gobgp_global_rib()

        paths = g1.get_adj_rib_out(q1, '30.0.0.0/24')
        self.assertEqual(len(paths), 1)
        self.assertNotIn('peer-id', paths[0])
        paths = g1.get_adj_rib_out(q2, '30.0.0.0/24')
        self.assertEqual(len(paths), 1)
        self.assertNotIn('peer-id', paths[0])

        g1.local('gobgp global rib del 30.0.0.0/24')

        def f():
            paths = g1.get_adj_rib_out(q1, '30.0.0.0/24')
            self.assertEqual(len(paths), 0)
            paths = g1.get_adj_rib_out(q2, '30.0.0.0/24')
            self.assertEqual(len(paths), 1)
            self.assertEqual(paths[0]['peer-id'], '192.168.0.2')

        assert_several_times(f)

    def test_19_check_grpc_add_neighbor(self):
        g1 = self.gobgp
        e1 = ExaBGPContainer(name='e1', asn=65000, router_id='192.168.0.7')
        time.sleep(e1.run())
        e1.add_peer(g1)
        self.quaggas['e1'] = e1
        n = e1.peers[g1]['local_addr'].split('/')[0]
        g1.local('gobgp n add {0} as 65000'.format(n))
        g1.add_peer(e1, reload_config=False)

        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=e1)

    def test_20_check_grpc_del_neighbor(self):
        g1 = self.gobgp
        e1 = self.quaggas['e1']
        n = e1.peers[g1]['local_addr'].split('/')[0]
        g1.local('gobgp n del {0}'.format(n))
        g1.del_peer(e1, reload_config=False)

    def test_21_check_withdrawal_2(self):
        g1 = self.gobgp
        g2 = self.quaggas['g2']

        prefix = '40.10.0.0/24'
        g1.add_route(prefix)
        wait_for_completion(lambda: len(g1.get_global_rib(prefix)) == 1)
        wait_for_completion(lambda: len(g2.get_global_rib(prefix)) == 1)

        r = g2.local('gobgp monitor global rib -j', stream=True, tty=False)

        g1.local('gobgp global rib del 40.10.0.0/24')
        del g1.routes[prefix]

        wait_for_completion(lambda: len(g1.get_global_rib(prefix)) == 0)
        wait_for_completion(lambda: len(g2.get_global_rib(prefix)) == 0)

        ret = json.loads(next(r))
        self.assertEqual(ret[0]['nlri']['prefix'], prefix)
        self.assertTrue('withdrawal' in ret[0])

    def test_22_check_cli_sorted(self):
        g1 = self.gobgp
        cnt = 0

        def next_prefix():
            for i in range(100, 105):
                for j in range(100, 105):
                    yield '{0}.{1}.0.0/24'.format(i, j)

        for p in next_prefix():
            g1.local('gobgp global rib add {0}'.format(p))
            cnt += 1

        cnt2 = 0
        g = next_prefix()
        n = next(g)
        for path in g1.local("gobgp global rib", capture=True).split('\n')[1:]:
            if [elem for elem in path.split(' ') if elem != ''][1] == n:
                try:
                    cnt2 += 1
                    n = next(g)
                except StopIteration:
                    break

        self.assertEqual(cnt, cnt2)

    def test_23_check_withdrawal3(self):
        gobgp_ctn_image_name = parser_option.gobgp_image
        g1 = self.gobgp
        g3 = GoBGPContainer(name='g3', asn=65006, router_id='192.168.0.8',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g4 = GoBGPContainer(name='g4', asn=65007, router_id='192.168.0.9',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)

        initial_wait_time = max(ctn.run() for ctn in [g3, g4])
        time.sleep(initial_wait_time)

        self.quaggas = {'g3': g3, 'g4': g4}

        g3.local('gobgp global rib add 50.0.0.0/24')

        g1.add_peer(g3, passive=True)
        g3.add_peer(g1)
        g1.add_peer(g4, passive=True)
        g4.add_peer(g1)

        self.test_01_neighbor_established()

        self.test_02_check_gobgp_global_rib()

        g4.local('gobgp global rib add 50.0.0.0/24 med 10')

        paths = g1.get_adj_rib_out(g3, '50.0.0.0/24')
        self.assertEqual(len(paths), 0)
        paths = g1.get_adj_rib_out(g4, '50.0.0.0/24')
        self.assertEqual(len(paths), 1)
        self.assertEqual(paths[0]['peer-id'], '192.168.0.8')

        g3.local('gobgp global rib del 50.0.0.0/24')

        paths = g1.get_adj_rib_out(g3, '50.0.0.0/24')
        self.assertEqual(len(paths), 1)
        self.assertEqual(paths[0]['peer-id'], '192.168.0.9')
        paths = g1.get_adj_rib_out(g4, '50.0.0.0/24')
        self.assertEqual(len(paths), 0)

    def test_24_add_quagga_md5(self):
        q6 = QuaggaBGPContainer(name='q6', asn=65004, router_id='192.168.0.5')
        self.quaggas['q6'] = q6
        initial_wait_time = q6.run()
        time.sleep(initial_wait_time)

        self.gobgp.add_peer(q6, passwd='password')
        q6.add_peer(self.gobgp, passwd='password')

        self.gobgp.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=q6)

    def test_25_add_quagga_md5_badpass(self):
        q7 = QuaggaBGPContainer(name='q6', asn=65004, router_id='192.168.0.7')
        self.quaggas['q7'] = q7
        initial_wait_time = q7.run()
        time.sleep(initial_wait_time)

        self.gobgp.add_peer(q7, passwd='password')
        q7.add_peer(self.gobgp, passwd='passwordBAD')

        time.sleep(10)
        state = self.gobgp.get_neighbor_state(q7)
        # should not establish
        self.assertEqual(state, BGP_FSM_ACTIVE)

    def test_26_dynamic_peer(self):
        gobgp_ctn_image_name = parser_option.gobgp_image
        g5 = GoBGPContainer(name='g5', asn=65008, router_id='192.168.0.8',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level,
                            bgp_config={
                                'peer-groups': [
                                    {
                                        'config':{
                                            'peer-group-name':'PG1',
                                            'peer-as': 65004,
                                            }
                                        }
                                    ],
                                'dynamic-neighbors':[
                                    {
                                        'config': {
                                            'prefix':'172.17.0.0/16',
                                            'peer-group':'PG1'
                                            }
                                        }
                                    
                                    ]
                                })
        q8 = QuaggaBGPContainer(name='q8', asn=65004, router_id='192.168.0.5')
        self.quaggas = { 'g8': q8 }

        time.sleep(g5.run())
        initial_wait_time = q8.run()
        time.sleep(initial_wait_time)

        q8.add_peer(g5)

        time.sleep(initial_wait_time)

        # should establish via the dynamic link
        # note that we are checking it on the Quagga side since GoBGP
        # config doesn't explicitly "know about" the peer
        q8.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g5)

    def test_27_dynamic_peer_md5(self):
        gobgp_ctn_image_name = parser_option.gobgp_image
        g6 = GoBGPContainer(name='g6', asn=65008, router_id='192.168.0.9',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level,
                            bgp_config={
                                'peer-groups': [
                                    {
                                        'config':{
                                            'peer-group-name':'PG1',
                                            'peer-as': 65004,
                                            'auth-password': 'password',
                                            }
                                        }
                                    ],
                                'dynamic-neighbors':[
                                    {
                                        'config': {
                                            'prefix':'172.17.0.0/16',
                                            'peer-group':'PG1'
                                            }
                                        }
                                    
                                    ]
                                })
        q9 = QuaggaBGPContainer(name='q9', asn=65004, router_id='192.168.0.10')
        self.quaggas = { 'g9': q9 }

        time.sleep(g6.run())
        initial_wait_time = q9.run()
        time.sleep(initial_wait_time)

        q9.add_peer(g6, passwd='password', passive=False)

        # should establish via the dynamic link
        # note that we are checking it on the Quagga side since GoBGP
        # config doesn't explicitly "know about" the peer
        q9.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g6, timeout=30)





# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import time

from lib.base import (
    BGPContainer,
    CmdBuffer,
    try_several_times,
    wait_for_completion,
    yellow,
    indent,
    local,
)


class BirdContainer(BGPContainer):

    WAIT_FOR_BOOT = 1
    SHARED_VOLUME = '/etc/bird'

    def __init__(self, name, asn, router_id, ctn_image_name='osrg/bird'):
        super(BirdContainer, self).__init__(name, asn, router_id,
                                            ctn_image_name)
        self.shared_volumes.append((self.config_dir, self.SHARED_VOLUME))

    def _start_bird(self):
        c = CmdBuffer()
        c << '#!/bin/sh'
        c << 'bird'
        cmd = 'echo "{0:s}" > {1}/start.sh'.format(c, self.config_dir)
        local(cmd)
        cmd = 'chmod 755 {0}/start.sh'.format(self.config_dir)
        local(cmd)
        self.local('{0}/start.sh'.format(self.SHARED_VOLUME))

    def _wait_for_boot(self):
        def _f():
            ret = self.local('birdc show status > /dev/null 2>&1; echo $?', capture=True)
            return ret == '0'

        return wait_for_completion(_f)

    def run(self):
        super(BirdContainer, self).run()
        self.reload_config()
        return self.WAIT_FOR_BOOT

    def create_config(self):
        c = CmdBuffer()
        c << 'router id {0};'.format(self.router_id)
        for peer, info in self.peers.items():
            c << 'protocol bgp {'
            c << '  local as {0};'.format(self.asn)
            n_addr = info['neigh_addr'].split('/')[0]
            c << '  neighbor {0} as {1};'.format(n_addr, peer.asn)
            c << '  multihop;'
            c << '}'

        with open('{0}/bird.conf'.format(self.config_dir), 'w') as f:
            print(yellow('[{0}\'s new bird.conf]'.format(self.name)))
            print(yellow(indent(str(c))))
            f.writelines(str(c))

    def reload_config(self):
        if len(self.peers) == 0:
            return

        def _reload():
            def _is_running():
                ps = self.local('ps', capture=True)
                running = False
                for line in ps.split('\n')[1:]:
                    if 'bird' in line:
                        running = True
                return running

            if _is_running():
                self.local('birdc configure')
            else:
                self._start_bird()

            self._wait_for_boot()
            if not _is_running():
                raise RuntimeError()

        try_several_times(_reload)


class RawBirdContainer(BirdContainer):
    def __init__(self, name, config, ctn_image_name='osrg/bird'):
        asn = None
        router_id = None
        for line in config.split('\n'):
            line = line.strip()
            if line.startswith('local as'):
                asn = int(line[len('local as'):].strip('; '))
            if line.startswith('router id'):
                router_id = line[len('router id'):].strip('; ')
        if not asn:
            raise Exception('asn not in bird config')
        if not router_id:
            raise Exception('router-id not in bird config')
        self.config = config
        super(RawBirdContainer, self).__init__(name, asn, router_id,
                                               ctn_image_name)

    def create_config(self):
        with open('{0}/bird.conf'.format(self.config_dir), 'w') as f:
            print(yellow('[{0}\'s new bird.conf]'.format(self.name)))
            print(yellow(indent(self.config)))
            f.writelines(self.config)



# Copyright (C) 2016 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from itertools import combinations
import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import BGP_FSM_ESTABLISHED, local
from lib.gobgp import GoBGPContainer


class GoBGPTestBase(unittest.TestCase):

    def assert_adv_count(self, src, dst, rf, count):
        self.assertEqual(count, len(src.get_adj_rib_out(dst, rf=rf)))
        self.assertEqual(count, len(dst.get_adj_rib_in(src, rf=rf)))

    def assert_upd_count(self, src, dst, sent, received):
        messages = src.get_neighbor(dst)['state']['messages']
        self.assertEqual(messages['sent'].get('update', 0), sent)
        self.assertEqual(messages['received'].get('update', 0), received)

    @classmethod
    def setUpClass(cls):
        # +----+              +----+
        # | g1 |----(iBGP)----| g2 |
        # +----+              +----+
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g2 = GoBGPContainer(name='g2', asn=65000, router_id='192.168.0.2',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)

        time.sleep(max(ctn.run() for ctn in [g1, g2]))

        g1.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g1.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        g2.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g2.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")

        g1.local("gobgp vrf vrf1 rib add 10.0.0.0/24")
        g1.local("gobgp vrf vrf2 rib add 10.0.0.0/24")
        g2.local("gobgp vrf vrf1 rib add 20.0.0.0/24")
        g2.local("gobgp vrf vrf3 rib add 20.0.0.0/24")

        cls.g1 = g1
        cls.g2 = g2

    # Test the problem which was reported on #1682
    # https://github.com/osrg/gobgp/issues/1682
    def test_01_neighbor_estabslihed_with_conflict_rtc_config(self):
        self.g1.add_peer(self.g2, vpn=True, passwd='rtc', graceful_restart=True)
        self.g2.add_peer(self.g1, vpn=False, passwd='rtc', graceful_restart=True)
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g2)

    # test each neighbor state is turned establish
    def test_02_neighbor_established(self):
        self.g2.update_peer(self.g1, vpn=True, passwd='rtc', graceful_restart=True)
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g2)

    def test_03_check_adj_rib(self):
        # VRF<#>  g1   g2
        #   1     (*)  (*)
        #   2     (*)
        #   3          (*)
        #
        # ( ): Empty VRF  (*): VRF with a route
        self.assert_adv_count(self.g1, self.g2, 'rtc', 2)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 1)

    def test_04_add_vrf(self):
        # VRF<#>  g1   g2
        #   1     (*)  (*)
        #   2     (*)
        #   3     ( )  (*)
        self.g1.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")
        time.sleep(1)

        self.assert_adv_count(self.g1, self.g2, 'rtc', 3)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 2)

    def test_05_add_route_on_vrf(self):
        # VRF<#>  g1   g2
        #   1     (*)  (*)
        #   2     (*)
        #   3     (*)  (*)
        self.g1.local("gobgp vrf vrf3 rib add 10.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g1, self.g2, 'rtc', 3)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 2)

    def test_06_del_route_on_vrf(self):
        # VRF<#>  g1   g2
        #   1     (*)  (*)
        #   2     (*)
        #   3     ( )  (*)
        self.g1.local("gobgp vrf vrf3 rib del 10.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g1, self.g2, 'rtc', 3)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 2)

    def test_07_del_vrf(self):
        # VRF<#>  g1   g2
        #   1     (*)  (*)
        #   2     (*)
        #   3          (*)
        self.g1.local("gobgp vrf del vrf3")
        time.sleep(1)

        self.assert_adv_count(self.g1, self.g2, 'rtc', 2)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 1)

    def test_08_del_vrf_with_route(self):
        # VRF<#>  g1   g2
        #   1          (*)
        #   2     (*)
        #   3          (*)
        self.g1.local("gobgp vrf del vrf1")
        time.sleep(1)

        self.assert_adv_count(self.g1, self.g2, 'rtc', 1)
        self.assert_adv_count(self.g1, self.g2, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g2, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g2, self.g1, 'ipv4-l3vpn', 0)

    def test_09_cleanup(self):
        self.g1.local("gobgp vrf del vrf2")
        self.g2.local("gobgp vrf del vrf1")
        self.g2.local("gobgp vrf del vrf3")

    def test_10_rr_setup(self):
        #               +------+
        #               |  g3  |
        #        +------| (RR) |------+
        #        |      +------+      |
        #      (iBGP)              (iBGP)
        #        |                    |
        # +-------------+      +-------------+
        # |     g4      |      |     g5      |
        # | (RR Client) |      | (RR Client) |
        # +-------------+      +-------------+
        gobgp_ctn_image_name = parser_option.gobgp_image
        g3 = GoBGPContainer(name='g3', asn=65000, router_id='192.168.0.3',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g4 = GoBGPContainer(name='g4', asn=65000, router_id='192.168.0.4',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g5 = GoBGPContainer(name='g5', asn=65000, router_id='192.168.0.5',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)

        time.sleep(max(ctn.run() for ctn in [g3, g4, g5]))

        g3.add_peer(g4, vpn=True, is_rr_client=True)
        g4.add_peer(g3, vpn=True)

        g3.add_peer(g5, vpn=True, is_rr_client=True)
        g5.add_peer(g3, vpn=True)

        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g4)
        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g5)

        self.__class__.g3 = g3
        self.__class__.g4 = g4
        self.__class__.g5 = g5

    def test_11_rr_check_adj_rib_from_rr(self):
        # VRF<#>  g3   g4   g5
        #   1          ( )  ( )
        #   2
        #   3
        self.g4.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        self.g5.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        time.sleep(1)

        # Check the counts of the sent UPDATE messages in order to detect the
        # infinite RTC UPDATE loop.
        # https://github.com/osrg/gobgp/issues/1630
        #
        # Do not forget to add RTC EOR:
        # RFC 4684 6
        # As a hint that initial RT membership exchange is complete,
        # implementations SHOULD generate an End-of-RIB marker, as defined in
        # [8], for the Route Target membership (afi, safi), regardless of
        # whether graceful-restart is enabled on the BGP session.
        self.assert_upd_count(self.g4, self.g3, sent=1+1, received=2+1)
        self.assert_upd_count(self.g5, self.g3, sent=1+1, received=2+1)

        def check_rtc(client):
            rib = self.g3.get_adj_rib_out(client, rf='rtc')
            self.assertEqual(1, len(rib))
            path = rib[0]
            self.assertEqual(self.g3.peers[client]['local_addr'].split('/')[0], path['nexthop'])
            ids = [attr['value'] for attr in path['attrs'] if attr['type'] == base.BGP_ATTR_TYPE_ORIGINATOR_ID]
            self.assertEqual(1, len(ids))
            self.assertEqual(self.g3.router_id, ids[0])

        check_rtc(self.g4)
        check_rtc(self.g5)

        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2
        #   3
        self.g4.local("gobgp vrf vrf1 rib add 40.0.0.0/24")
        self.g5.local("gobgp vrf vrf1 rib add 50.0.0.0/24")
        time.sleep(1)

        def check_ipv4_l3vpn(client):
            rib = self.g3.get_adj_rib_out(client, rf='ipv4-l3vpn')
            self.assertEqual(1, len(rib))
            path = rib[0]
            self.assertNotEqual(self.g3.peers[client]['local_addr'].split('/')[0], path['nexthop'])
            ids = [attr['value'] for attr in path['attrs'] if attr['type'] == base.BGP_ATTR_TYPE_ORIGINATOR_ID]
            self.assertEqual(1, len(ids))
            self.assertNotEqual(client.router_id, ids[0])

        check_ipv4_l3vpn(self.g4)
        check_ipv4_l3vpn(self.g5)

    def test_12_rr_add_vrf(self):
        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2          ( )
        #   3
        self.g4.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 1)

    def test_13_rr_add_route_on_vrf(self):
        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2          (*)
        #   3
        self.g4.local("gobgp vrf vrf2 rib add 40.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 1)

    def test_14_rr_del_vrf_with_route(self):
        # VRF<#>  g3   g4   g5
        #   1               (*)
        #   2          (*)
        #   3
        self.g4.local("gobgp vrf del vrf1")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 0)

    def test_15_rr_cleanup(self):
        self.g4.local("gobgp vrf del vrf2")
        self.g5.local("gobgp vrf del vrf1")

    def test_20_rr_and_non_rr_setup(self):
        # +----------+            +----------+
        # |    g1    |---(iBGP)---|    g2    |
        # | (Non RR  |            | (Non RR  |
        # |  Client) |            |  Client) |
        # +----------+            +----------+
        #      |                        |
        #      +--(iBGP)--+  +--(iBGP)--+
        #                 |  |
        #               +------+
        #               |  g3  |
        #        +------| (RR) |------+
        #        |      +------+      |
        #      (iBGP)              (iBGP)
        #        |                    |
        # +-------------+      +-------------+
        # |     g4      |      |     g5      |
        # | (RR Client) |      | (RR Client) |
        # +-------------+      +-------------+
        self.g3.add_peer(self.g1, vpn=True)
        self.g1.add_peer(self.g3, vpn=True)
        self.g3.add_peer(self.g2, vpn=True)
        self.g2.add_peer(self.g3, vpn=True)

        self.g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g1)
        self.g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g2)

    def test_21_rr_and_non_rr_add_vrf_on_rr_clients(self):
        # VRF<#>  g1   g2   g3   g4   g5
        #   1                    (*)  (*)
        #   2
        #   3
        self.g4.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        self.g5.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        self.g4.local("gobgp vrf vrf1 rib add 40.0.0.0/24")
        self.g5.local("gobgp vrf vrf1 rib add 50.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g1, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g1, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g1, self.g3, 'rtc', 0)
        self.assert_adv_count(self.g1, self.g3, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

    def test_22_rr_and_non_rr_add_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5
        #   1     (*)            (*)  (*)
        #   2
        #   3
        self.g1.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        self.g1.local("gobgp vrf vrf1 rib add 10.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g1, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g1, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g2, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g2, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g1, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g1, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g3, 'rtc', 0)
        self.assert_adv_count(self.g2, self.g3, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 2)

    def test_23_rr_and_non_rr_add_another_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5
        #   1     (*)            (*)  (*)
        #   2          (*)
        #   3
        self.g2.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        self.g2.local("gobgp vrf vrf2 rib add 20.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g1, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g1, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g2, 'rtc', 1)
        self.assert_adv_count(self.g3, self.g2, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g1, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g1, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g2, self.g3, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 2)

    def test_24_rr_and_non_rr_add_another_vrf_on_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5
        #   1     (*)            (*)  (*)
        #   2          (*)       (*)
        #   3
        self.g4.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        self.g4.local("gobgp vrf vrf2 rib add 40.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g1, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g1, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g2, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g2, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g1, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g1, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g2, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g2, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 3)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 2)

    def test_25_rr_and_non_rr_clenup(self):
        self.g1.local("gobgp vrf del vrf1")
        self.g2.local("gobgp vrf del vrf2")
        self.g4.local("gobgp vrf del vrf1")
        self.g4.local("gobgp vrf del vrf2")
        self.g5.local("gobgp vrf del vrf1")

    def test_30_ebgp_setup(self):
        # +----+              +----+
        # | g6 |----(eBGP)----| g7 |
        # +----+              +----+
        gobgp_ctn_image_name = parser_option.gobgp_image
        g6 = GoBGPContainer(name='g6', asn=65001, router_id='192.168.0.6',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g7 = GoBGPContainer(name='g7', asn=65002, router_id='192.168.0.7',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)

        time.sleep(max(ctn.run() for ctn in [g6, g7]))

        g6.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g6.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        g7.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g7.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")

        g6.local("gobgp vrf vrf1 rib add 60.0.0.0/24")
        g6.local("gobgp vrf vrf2 rib add 60.0.0.0/24")
        g7.local("gobgp vrf vrf1 rib add 70.0.0.0/24")
        g7.local("gobgp vrf vrf3 rib add 70.0.0.0/24")

        for a, b in combinations([g6, g7], 2):
            a.add_peer(b, vpn=True, passwd='rtc', graceful_restart=True)
            b.add_peer(a, vpn=True, passwd='rtc', graceful_restart=True)

        g6.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g7)

        self.__class__.g6 = g6
        self.__class__.g7 = g7

    def test_31_ebgp_check_adj_rib(self):
        # VRF<#>  g6   g7
        #   1     (*)  (*)
        #   2     (*)
        #   3          (*)
        self.assert_adv_count(self.g6, self.g7, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 1)

    def test_32_ebgp_add_vrf(self):
        # VRF<#>  g6   g7
        #   1     (*)  (*)
        #   2     (*)
        #   3     ( )  (*)
        self.g6.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")
        time.sleep(1)

        self.assert_adv_count(self.g6, self.g7, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 2)

    def test_33_ebgp_add_route_on_vrf(self):
        # VRF<#>  g6   g7
        #   1     (*)  (*)
        #   2     (*)
        #   3     (*)  (*)
        self.g6.local("gobgp vrf vrf3 rib add 60.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g6, self.g7, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 2)

    def test_34_ebgp_del_route_on_vrf(self):
        # VRF<#>  g6   g7
        #   1     (*)  (*)
        #   2     (*)
        #   3     ( )  (*)
        self.g6.local("gobgp vrf vrf3 rib del 60.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g6, self.g7, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 2)

    def test_35_ebgp_del_vrf(self):
        # VRF<#>  g6   g7
        #   1     (*)  (*)
        #   2     (*)
        #   3          (*)
        self.g6.local("gobgp vrf del vrf3")
        time.sleep(1)

        self.assert_adv_count(self.g6, self.g7, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 1)

    def test_36_ebgp_del_vrf_with_route(self):
        # VRF<#>  g6   g7
        #   1          (*)
        #   2     (*)
        #   3          (*)
        self.g6.local("gobgp vrf del vrf1")
        time.sleep(1)

        self.assert_adv_count(self.g6, self.g7, 'rtc', 1)
        self.assert_adv_count(self.g6, self.g7, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g7, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g7, self.g6, 'ipv4-l3vpn', 0)

    def test_37_ebgp_cleanup(self):
        self.g6.local("gobgp vrf del vrf2")
        self.g7.local("gobgp vrf del vrf1")
        self.g7.local("gobgp vrf del vrf3")

    def test_40_ibgp_ebgp_setup(self):
        # +----------+            +----------+
        # |    g1    |---(iBGP)---|    g2    |
        # | (Non RR  |            | (Non RR  |
        # |  Client) |            |  Client) |
        # +----------+            +----------+
        #      |                        |
        #      +--(iBGP)--+  +--(iBGP)--+
        #                 |  |
        #               +------+                    +----+              +----+
        #               |  g3  |----------(eBGP)----| g6 |----(eBGP)----| g7 |
        #        +------| (RR) |------+             +----+              +----+
        #        |      +------+      |
        #      (iBGP)              (iBGP)
        #        |                    |
        # +-------------+      +-------------+
        # |     g4      |      |     g5      |
        # | (RR Client) |      | (RR Client) |
        # +-------------+      +-------------+
        g1, g2, g3, g4, g5, g6, g7 = (
            self.g1, self.g2, self.g3, self.g4, self.g5, self.g6, self.g7)
        g3.add_peer(g6, vpn=True)
        g6.add_peer(g3, vpn=True)

        g1.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g2.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        # g3 (RR)
        g4.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g5.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        g6.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        g7.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")

        g1.local("gobgp vrf vrf1 rib add 10.0.0.0/24")
        g2.local("gobgp vrf vrf2 rib add 20.0.0.0/24")
        # g3 (RR)
        g4.local("gobgp vrf vrf1 rib add 40.0.0.0/24")
        g5.local("gobgp vrf vrf2 rib add 50.0.0.0/24")
        g6.local("gobgp vrf vrf1 rib add 60.0.0.0/24")
        g7.local("gobgp vrf vrf3 rib add 70.0.0.0/24")

        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g6)

    def test_41_ibgp_ebgp_check_adj_rib(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                             |      (*)
        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_42_01_ibgp_ebgp_add_vrf_on_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                         ( ) |      (*)
        self.g5.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_42_02_ibgp_ebgp_add_route_on_vrf_on_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                         (*) |      (*)
        self.g5.local("gobgp vrf vrf3 rib add 50.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 3)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_42_03_ibgp_ebgp_del_route_on_vrf_on_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                         ( ) |      (*)
        self.g5.local("gobgp vrf vrf3 rib del 50.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_42_04_ibgp_ebgp_del_vrf_on_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                             |      (*)
        self.g5.local("gobgp vrf del vrf3")
        time.sleep(2)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_43_01_ibgp_ebgp_add_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3     ( )                     |      (*)
        self.g1.local("gobgp vrf add vrf3 rd 300:300 rt both 300:300")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_43_02_ibgp_ebgp_add_route_on_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3     (*)                     |      (*)
        self.g1.local("gobgp vrf vrf3 rib add 10.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 3)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_43_03_ibgp_ebgp_del_route_on_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3     ( )                     |      (*)
        self.g1.local("gobgp vrf vrf3 rib del 10.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 3)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_43_04_ibgp_ebgp_del_vrf_on_non_rr_client(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                             |      (*)
        self.g1.local("gobgp vrf del vrf3")
        time.sleep(2)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_44_01_ibgp_ebgp_add_vrf_on_ebgp_peer(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |      ( )
        #   3                             |      (*)
        self.g7.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 4)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_44_02_ibgp_ebgp_add_route_on_ebgp_peer(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |      (*)
        #   3                             |      (*)
        self.g7.local("gobgp vrf vrf2 rib add 70.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 4)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 2)

    def test_44_03_ibgp_ebgp_del_route_on_vrf_on_ebgp_peer(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |      ( )
        #   3                             |      (*)
        self.g7.local("gobgp vrf vrf2 rib del 70.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 4)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 3)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_44_04_ibgp_ebgp_del_vrf_on_ebgp_peer(self):
        # VRF<#>  g1   g2   g3   g4   g5  | g6   g7
        #   1     (*)            (*)      | (*)
        #   2          (*)            (*) |
        #   3                             |      (*)
        self.g7.local("gobgp vrf del vrf2")
        time.sleep(2)

        self.assert_adv_count(self.g3, self.g6, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g6, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g6, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g6, self.g3, 'ipv4-l3vpn', 1)

    def test_45_ibgp_ebgp_cleanup(self):
        self.g1.local("gobgp vrf del vrf1")
        self.g2.local("gobgp vrf del vrf2")
        self.g4.local("gobgp vrf del vrf1")
        self.g5.local("gobgp vrf del vrf2")
        self.g6.local("gobgp vrf del vrf1")
        self.g7.local("gobgp vrf del vrf3")

    def test_50_rr_addpath_setup(self):
        # Test cases for the infinite RTC UPDATE message loop:
        # https://github.com/osrg/gobgp/issues/1688#issuecomment-391456615
        # https://github.com/osrg/gobgp/pull/1703

        #               +------+
        #               |  g3  |
        #        +------| (RR) |------+
        #        |      +------+      |
        # (iBGP+Add-Paths)     (iBGP+Add-Paths)
        #        |                    |
        # +-------------+      +-------------+
        # |     g4      |      |     g5      |
        # | (RR Client) |      | (RR Client) |
        # +-------------+      +-------------+
        g3, g4, g5 = self.g3, self.g4, self.g5

        g3.update_peer(g4, vpn=True, addpath=16, is_rr_client=True)
        g4.update_peer(g3, vpn=True, addpath=16)

        g3.update_peer(g5, vpn=True, addpath=16, is_rr_client=True)
        g5.update_peer(g3, vpn=True, addpath=16)

        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g4)
        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g5)

    def test_51_rr_addpath_check_adj_rib_from_rr(self):
        # VRF<#>  g3   g4   g5
        #   1          ( )  ( )
        #   2
        #   3
        self.g4.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        self.g5.local("gobgp vrf add vrf1 rd 100:100 rt both 100:100")
        time.sleep(1)

        # Check the counts of the sent UPDATE messages in order to detect the
        # infinite RTC UPDATE loop.
        #
        # Do not forget to add RTC EOR:
        # RFC 4684 6
        # As a hint that initial RT membership exchange is complete,
        # implementations SHOULD generate an End-of-RIB marker, as defined in
        # [8], for the Route Target membership (afi, safi), regardless of
        # whether graceful-restart is enabled on the BGP session.
        self.assert_upd_count(self.g4, self.g3, sent=1+1, received=2+1)
        self.assert_upd_count(self.g5, self.g3, sent=1+1, received=2+1)

        def check_rtc(client):
            rib = self.g3.get_adj_rib_out(client, rf='rtc')
            self.assertEqual(1, len(rib))
            path = rib[0]
            self.assertEqual(self.g3.peers[client]['local_addr'].split('/')[0], path['nexthop'])
            ids = [attr['value'] for attr in path['attrs'] if attr['type'] == base.BGP_ATTR_TYPE_ORIGINATOR_ID]
            self.assertEqual(1, len(ids))
            self.assertEqual(self.g3.router_id, ids[0])

        check_rtc(self.g4)
        check_rtc(self.g5)

        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2
        #   3
        self.g4.local("gobgp vrf vrf1 rib add 40.0.0.0/24")
        self.g5.local("gobgp vrf vrf1 rib add 50.0.0.0/24")
        time.sleep(1)

        def check_ipv4_l3vpn(client):
            rib = self.g3.get_adj_rib_out(client, rf='ipv4-l3vpn')
            self.assertEqual(1, len(rib))
            path = rib[0]
            self.assertNotEqual(self.g3.peers[client]['local_addr'].split('/')[0], path['nexthop'])
            ids = [attr['value'] for attr in path['attrs'] if attr['type'] == base.BGP_ATTR_TYPE_ORIGINATOR_ID]
            self.assertEqual(1, len(ids))
            self.assertNotEqual(client.router_id, ids[0])

        check_ipv4_l3vpn(self.g4)
        check_ipv4_l3vpn(self.g5)

    def test_52_rr_addpath_add_vrf(self):
        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2          ( )
        #   3
        self.g4.local("gobgp vrf add vrf2 rd 200:200 rt both 200:200")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 1)

    def test_53_rr_addpath_add_route_on_vrf(self):
        # VRF<#>  g3   g4   g5
        #   1          (*)  (*)
        #   2          (*)
        #   3
        self.g4.local("gobgp vrf vrf2 rib add 40.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 2)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 1)

    def test_54_rr_addpath_del_route_on_vrf(self):
        # Related issue:
        # https://github.com/osrg/gobgp/issues/1688

        # VRF<#>  g3   g4   g5
        #   1          ( )  (*)
        #   2          (*)
        #   3
        self.g4.local("gobgp vrf vrf1 rib del 40.0.0.0/24")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 2)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 0)

    def test_55_rr_addpath_del_vrf_with_route(self):
        # VRF<#>  g3   g4   g5
        #   1               (*)
        #   2          (*)
        #   3
        self.g4.local("gobgp vrf del vrf1")
        time.sleep(1)

        self.assert_adv_count(self.g4, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g4, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g4, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g4, 'ipv4-l3vpn', 0)

        self.assert_adv_count(self.g5, self.g3, 'rtc', 1)
        self.assert_adv_count(self.g5, self.g3, 'ipv4-l3vpn', 1)

        self.assert_adv_count(self.g3, self.g5, 'rtc', 2)
        self.assert_adv_count(self.g3, self.g5, 'ipv4-l3vpn', 0)

    def test_56_rr_addpath_cleanup(self):
        self.g4.local("gobgp vrf del vrf2")
        self.g5.local("gobgp vrf del vrf1")





#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import grpc

import gobgp_pb2
import gobgp_pb2_grpc
import attribute_pb2
import common_pb2
import nlri_pb2

_TIMEOUT_SECONDS = 1000


def run():
    channel = grpc.insecure_channel('localhost:50051')
    stub = gobgp_pb2_grpc.GoBgpServiceStub(channel)

    nlri = nlri_pb2.NLRI(
        prefix=nlri_pb2.IPAddressPrefix(
            prefix_len=24,
            prefix="10.0.0.0",
        )
    )

    as_segment = attribute_pb2.AsSegment(
        type=attribute_pb2.AsSegment.TYPE_AS_SEQUENCE,
        numbers=[100, 200],
    )

    attributes = [
        attribute_pb2.Attribute(
            origin=attribute_pb2.OriginAttribute(origin=2)  # ORIGIN_INCOMPLETE
        ),
        attribute_pb2.Attribute(
            as_path=attribute_pb2.AsPathAttribute(segments=[as_segment])
        ),
        attribute_pb2.Attribute(
            next_hop=attribute_pb2.NextHopAttribute(next_hop="1.1.1.1")
        ),
    ]

    stub.AddPath(
        gobgp_pb2.AddPathRequest(
            table_type=gobgp_pb2.TABLE_TYPE_GLOBAL,
            path=gobgp_pb2.Path(
                nlri=nlri,
                pattrs=attributes,
                family=common_pb2.Family(
                    afi=common_pb2.Family.AFI_IP,
                    safi=common_pb2.Family.SAFI_UNICAST,
                ),
            )
        ),
        timeout=_TIMEOUT_SECONDS,
    )

if __name__ == '__main__':
    run()



# Copyright (C) 2016 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from itertools import chain
import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    BGP_FSM_ACTIVE,
    BGP_FSM_IDLE,
    BGP_FSM_ESTABLISHED,
    LONG_LIVED_GRACEFUL_RESTART_TIME,
    local,
)
from lib.gobgp import GoBGPContainer


class GoBGPTestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65000, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g2 = GoBGPContainer(name='g2', asn=65001, router_id='192.168.0.2',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g3 = GoBGPContainer(name='g3', asn=65002, router_id='192.168.0.3',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        g4 = GoBGPContainer(name='g4', asn=65003, router_id='192.168.0.4',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        ctns = [g1, g2, g3, g4]

        initial_wait_time = max(ctn.run() for ctn in ctns)

        time.sleep(initial_wait_time)

        g1.add_peer(g2, graceful_restart=True, llgr=True)
        g2.add_peer(g1, graceful_restart=True, llgr=True)
        g1.add_peer(g3, graceful_restart=True, llgr=True)
        g3.add_peer(g1, graceful_restart=True, llgr=True)
        g1.add_peer(g4, graceful_restart=True)
        g4.add_peer(g1, graceful_restart=True)

        cls.bgpds = {'g1': g1, 'g2': g2, 'g3': g3, 'g4': g4}

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']
        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g2)
        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g3)
        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g4)

    def _test_graceful_restart(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']

        g1.wait_for(expected_state=BGP_FSM_ACTIVE, peer=g2)

        time.sleep(1)

        self.assertEqual(len(g1.get_global_rib('10.0.0.0/24')), 1)
        # check llgr-stale community is added to 10.0.0.0/24
        r = g1.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertTrue(0xffff0006 in comms)

        # 10.10.0.0/24 is announced with no-llgr community
        # must not exist in the rib
        self.assertEqual(len(g1.get_global_rib('10.10.0.0/24')), 0)
        for d in g1.get_global_rib():
            for p in d['paths']:
                self.assertTrue(p['stale'])

        # check llgr-stale community is present in received route 10.0.0.0/24
        self.assertEqual(len(g3.get_global_rib('10.0.0.0/24')), 1)
        r = g3.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertTrue(0xffff0006 in comms)

        # g4 is not llgr capable, llgr-stale route must be
        # withdrawn
        self.assertEqual(len(g4.get_global_rib('10.0.0.0/24')), 0)

    def test_02_hold_timer_expiry_graceful_restart(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']

        g2.local('gobgp global rib add 10.0.0.0/24')
        g2.local('gobgp global rib add 10.10.0.0/24 community no-llgr')

        time.sleep(1)

        g2.local("ip route add blackhole {}/32".format(g1.ip_addrs[0][1].split("/")[0]))
        g2.local("ip route add blackhole {}/32".format(g3.ip_addrs[0][1].split("/")[0]))
        g2.local("ip route add blackhole {}/32".format(g4.ip_addrs[0][1].split("/")[0]))

        self._test_graceful_restart()

    def test_03_neighbor_established_after_hold_time_expiry(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']
        g2.local("ip route del blackhole {}/32".format(g1.ip_addrs[0][1].split("/")[0]))
        g2.local("ip route del blackhole {}/32".format(g3.ip_addrs[0][1].split("/")[0]))
        g2.local("ip route del blackhole {}/32".format(g4.ip_addrs[0][1].split("/")[0]))

        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g2)
        time.sleep(1)
        self.assertEqual(len(g1.get_global_rib('10.0.0.0/24')), 1)
        self.assertEqual(len(g1.get_global_rib('10.10.0.0/24')), 1)
        # check llgr-stale community is not present in 10.0.0.0/24
        r = g1.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertFalse(0xffff0006 in comms)

        # check llgr-stale community is not present in 10.0.0.0/24
        self.assertEqual(len(g3.get_global_rib('10.0.0.0/24')), 1)
        r = g3.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertFalse(0xffff0006 in comms)

    def test_04_graceful_restart(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']

        g2.local('gobgp global rib add 10.0.0.0/24')
        g2.local('gobgp global rib add 10.10.0.0/24 community no-llgr')

        time.sleep(1)

        g2.stop_gobgp()
        self._test_graceful_restart()

    def test_05_softreset_preserves_llgr_community(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']

        g1.softreset(g2)
        time.sleep(1)

        # 10.10.0.0/24 received with no-llgr community is not reinstalled to rib
        self.assertEqual(len(g1.get_global_rib('10.10.0.0/24')), 0)
        # Stale flags are not cleared
        for d in g1.get_global_rib():
            for p in d['paths']:
                self.assertTrue(p['stale'])

        # check llgr-stale community is not cleared from 10.0.0.0/24
        r = g1.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertTrue(0xffff0006 in comms)

        # check llgr-stale community is not cleared in route 10.0.0.0/24 on g3
        self.assertEqual(len(g3.get_global_rib('10.0.0.0/24')), 1)
        r = g3.get_global_rib('10.0.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertTrue(0xffff0006 in comms)

        # g4 is not llgr capable, llgr-stale route must not be advertized on softreset g1
        self.assertEqual(len(g4.get_global_rib('10.0.0.0/24')), 0)

    def test_06_neighbor_established(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        # g3 = self.bgpds['g3']
        # g4 = self.bgpds['g4']

        g2.start_gobgp(graceful_restart=True)
        g2.local('gobgp global rib add 10.0.0.0/24')
        g2.local('gobgp global rib add 10.10.0.0/24')

        g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g2)
        time.sleep(1)
        self.assertEqual(len(g1.get_global_rib('10.0.0.0/24')), 1)
        self.assertEqual(len(g1.get_global_rib('10.10.0.0/24')), 1)
        for d in g1.get_global_rib():
            for p in d['paths']:
                self.assertFalse(p.get('stale', False))

    def test_07_llgr_stale_route_depreferenced(self):
        g1 = self.bgpds['g1']
        g2 = self.bgpds['g2']
        g3 = self.bgpds['g3']
        g4 = self.bgpds['g4']
        g4.local('gobgp global rib add 10.0.0.0/24 med 100')
        time.sleep(1)
        # check g2's path is chosen as best and advertised
        rib = g3.get_global_rib('10.0.0.0/24')
        self.assertEqual(len(rib), 1)
        self.assertTrue(g2.asn in rib[0]['paths'][0]['aspath'])

        g2.stop_gobgp()
        g1.wait_for(expected_state=BGP_FSM_ACTIVE, peer=g2)

        time.sleep(1)

        # llgr_stale route depreference must happen
        # check g4's path is chosen as best and advertised
        rib = g3.get_global_rib('10.0.0.0/24')
        self.assertEqual(len(rib), 1)
        self.assertTrue(g4.asn in rib[0]['paths'][0]['aspath'])

        # if no candidate exists, llgr_stale route will be chosen as best
        rib = g3.get_global_rib('10.10.0.0/24')
        self.assertEqual(len(rib), 1)
        self.assertTrue(g2.asn in rib[0]['paths'][0]['aspath'])

    def test_08_llgr_restart_timer_expire(self):
        time.sleep(LONG_LIVED_GRACEFUL_RESTART_TIME + 5)
        g3 = self.bgpds['g3']
        rib = g3.get_global_rib('10.10.0.0/24')
        self.assertEqual(len(rib), 0)

    def test_09_peer_disabled_during_graceful_restart(self):
        g1 = self.bgpds['g1']
        g3 = self.bgpds['g3']

        g3.local('gobgp global rib add 10.20.0.0/24')

        time.sleep(1)

        g3.local("ip route add blackhole {}/32".format(g1.ip_addrs[0][1].split("/")[0]))
        time.sleep(1)
        # disable peering after traffic is blocked
        g3.local("gobgp nei {} disable".format(g1.ip_addrs[0][1].split("/")[0]))

        # wait for hold timer and unblock traffic
        g1.wait_for(expected_state=BGP_FSM_ACTIVE, peer=g3)
        g3.local("ip route del blackhole {}/32".format(g1.ip_addrs[0][1].split("/")[0]))

        g1.wait_for(expected_state=BGP_FSM_ACTIVE, peer=g3)

        self.assertEqual(len(g1.get_global_rib('10.20.0.0/24')), 1)
        r = g1.get_global_rib('10.20.0.0/24')[0]['paths'][0]
        comms = list(chain.from_iterable([attr['communities'] for attr in r['attrs'] if attr['type'] == 8]))
        self.assertEqual(comms.count(0xffff0006), 1)





# Copyright (C) 2017 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import time
import unittest

import collections
collections.Callable = collections.abc.Callable


from lib.noseplugin import parser_option

from lib import base
from lib.base import (
    BGP_FSM_ESTABLISHED,
    assert_several_times,
    local,
)
from lib.gobgp import GoBGPContainer
from lib.quagga import QuaggaBGPContainer


class GoBGPTestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        gobgp_ctn_image_name = parser_option.gobgp_image
        base.TEST_PREFIX = parser_option.test_prefix

        g1 = GoBGPContainer(name='g1', asn=65001, router_id='192.168.0.1',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        q1 = QuaggaBGPContainer(name='q1', asn=65002, router_id='192.168.0.2')
        g2 = GoBGPContainer(name='g2', asn=65001, router_id='192.168.0.3',
                            ctn_image_name=gobgp_ctn_image_name,
                            log_level=parser_option.gobgp_log_level)
        ctns = [g1, g2, q1]

        initial_wait_time = max(ctn.run() for ctn in ctns)

        time.sleep(initial_wait_time)

        g1.add_peer(q1)
        q1.add_peer(g1)

        q1.add_peer(g2)
        g2.add_peer(q1)

        g1.add_route('10.0.0.0/24')

        cls.g1 = g1
        cls.g2 = g2
        cls.q1 = q1
        cls.ctns = {n.name: n for n in ctns}

    # test each neighbor state is turned establish
    def test_01_neighbor_established(self):
        self.g1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.q1)
        self.q1.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.g2)

    def test_02_check_reject_as_loop(self):
        def f():
            r = self.g2.get_neighbor(self.q1)
            self.assertTrue('afi_safis' in r)
            received = 0
            for afisafi in r['afi_safis']:
                self.assertTrue('state' in afisafi)
                s = afisafi.get('state')
                self.assertTrue('received' in s)
                received += s.get('received')
                # hacky. 'accepted' is zero so the key was deleted due to
                # omitempty tag in bgp_configs.go.
                self.assertFalse(s.get('accepted'), None)
            self.assertEqual(received, 1)

        assert_several_times(f)

    def test_03_update_peer(self):
        self.g2.update_peer(self.q1, allow_as_in=10)

        self.g2.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=self.q1)

    def test_04_check_accept_as_loop(self):
        def f():
            r = self.g2.get_neighbor(self.q1)
            self.assertTrue('afi_safis' in r)
            received = 0
            accepted = 0
            for afisafi in r['afi_safis']:
                self.assertTrue('state' in afisafi)
                s = afisafi.get('state')
                self.assertTrue('received' in s)
                received += s.get('received')
                accepted += s.get('accepted')
            self.assertEqual(received, 1)
            self.assertEqual(accepted, 1)

        assert_several_times(f)

    def test_05_check_remove_private_as_peer_all(self):
        g3 = GoBGPContainer(name='g3', asn=100, router_id='192.168.0.4',
                            ctn_image_name=parser_option.gobgp_image,
                            log_level=parser_option.gobgp_log_level)
        g4 = GoBGPContainer(name='g4', asn=200, router_id='192.168.0.5',
                            ctn_image_name=parser_option.gobgp_image,
                            log_level=parser_option.gobgp_log_level)
        time.sleep(max(ctn.run() for ctn in [g3, g4]))

        self.ctns['g3'] = g3
        self.ctns['g4'] = g4

        self.g2.add_peer(g3)
        g3.add_peer(self.g2)

        g3.add_peer(g4, remove_private_as='all')
        g4.add_peer(g3)

        self.g2.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g3)
        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g4)

        def f():
            rib = g4.get_global_rib()
            self.assertEqual(len(rib), 1)
            self.assertEqual(len(rib[0]['paths']), 1)
            self.assertEqual(rib[0]['paths'][0]['aspath'], [100])

        assert_several_times(f)

    def test_06_check_remove_private_as_peer_replace(self):
        g3 = self.ctns['g3']
        g4 = self.ctns['g4']
        g3.update_peer(g4, remove_private_as='replace')

        g3.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g4)

        def f():
            rib = g4.get_global_rib()
            self.assertEqual(rib[0]['paths'][0]['aspath'], [100, 100, 100, 100])

        assert_several_times(f)

    def test_07_check_replace_peer_as(self):
        g5 = GoBGPContainer(name='g5', asn=100, router_id='192.168.0.6',
                            ctn_image_name=parser_option.gobgp_image,
                            log_level=parser_option.gobgp_log_level)
        time.sleep(g5.run())

        g4 = self.ctns['g4']
        g4.add_peer(g5, replace_peer_as=True)
        g5.add_peer(g4)

        g4.wait_for(expected_state=BGP_FSM_ESTABLISHED, peer=g5)

        def f():
            rib = g5.get_global_rib()
            self.assertEqual(rib[0]['paths'][0]['aspath'], [200, 200, 200, 200, 200])

        assert_several_times(f)


